#ifndef CKCoreAltitude_h
#define CKCoreAltitude_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <CoreMotion/CoreMotion.h>

typedef NS_ENUM(NSInteger, CKCoreAltitudeAuthorization)
  {
  CKCoreAltitudeAuthorizedAlways      = 0,
  CKCoreAltitudeDenied                = 1,
  CKCoreAltitudeRequest               = 2,
  CKCoreAltitudeRestricted            = 3,
  };

//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreAltitudeData: NSObject

@property(nonatomic) double relativeAltitude;
@property(nonatomic) double pressure;
@property(nonatomic) double verticalSpeed;

@end

//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreAbsoluteAltitudeData : NSObject

@property(nonatomic) double altitude;
@property(nonatomic) double accuracy;
@property(nonatomic) double precision;

@end

//-------------------------------------------------------------------------------------------------------------------------
// TYPEDEFS
//-------------------------------------------------------------------------------------------------------------------------
typedef void (^CKCoreRelativeAltitudeChangedCallback) (CKCoreAltitudeData *newAltitude, NSError *error);
typedef void (^CKCoreAbsoluteAltitudeChangedCallback) (CKCoreAbsoluteAltitudeData *newAltitude, NSError *error);

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreAltitude: NSObject

@property (nonatomic, readonly) Boolean                     isRelativeSensorAvailable;
@property (nonatomic, readonly) Boolean                     isAbsoluteSensorAvailable;
@property (nonatomic, readonly) CKCoreAltitudeAuthorization isAuthorized;

- (void)                        cleanUp;
- (void)                        startRelativeAltitudeScan:(CKCoreRelativeAltitudeChangedCallback) completionBlock;
- (void)                        stopRelativeAltitudeScan;
- (void)                        startAbsoluteAltitudeScan:(CKCoreAbsoluteAltitudeChangedCallback) completionBlock;
- (void)                        stopAbsoluteAltitudeScan;

@end

#endif 
