
/**
 CKCoreAnthropic.h
 
 This header file contains the interface for the CKCoreAnthropic class, which provides methods for interacting with the Anthropic AI chat platform.
 
 */

#ifndef CKCoreAnthropic_h
#define CKCoreAnthropic_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

/**
 CKCoreAnthropicStreamCallback
 
 A block that is called when a streaming request is completed, providing the response data and any errors.
 
 @param task The NSURLSessionDataTask object that was used to make the request.
 @param data The response data received from the server.
 @param error Any errors that occurred during the request.
 */

typedef void (^CKCoreAnthropicStreamCallback) (NSURLSessionDataTask *task, NSData *data, NSError *error);

//----------------------------------------------------------------------------------------------------
/**
 CKCoreAnthropic
 
 The CKCoreAnthropic class provides methods for interacting with the Anthropic AI chat platform.
 
 */

@interface CKCoreAnthropic: NSObject

/**
 promptChatCompletion:withModel:withMaxTokens:withPrompt:completionBlock:
 
 Sends a prompt to the Anthropic AI chat platform and returns the response, along with any errors and the time it took to receive the response.
 
 @param apiKey The API key for the Anthropic AI platform.
 @param withModel The model to use for generating the response. Can be one of the predefined models or a custom model.
 @param withMaxTokens The maximum number of tokens to generate in the response.
 @param withPrompt The prompt to send to the platform.
 @param completionBlock A block that is called when the request is completed, providing the response, raw response data, any errors, and the time it took to receive the response.
 */

+ (void) promptChatCompletion:(NSString *) apiKey withModel:(NSString *) withModel withMaxTokens:(NSInteger) withMaxTokens withPrompt:(NSString *) withPrompt completionBlock:(void (^)(NSString *response, NSDictionary *rawResponse, NSError *error, NSTimeInterval opDelay)) completionBlock;

@end

/**
 ANTHROPICMODEL_Opus
 
 A predefined model for generating responses in the form of Opus poems.
 */

#define ANTHROPICMODEL_Opus    @"claude-3-opus-20240229"

/**
 ANTHROPICMODEL_Sonnet
 
 A predefined model for generating responses in the form of sonnets.
 */

#define ANTHROPICMODEL_Sonnet  @"claude-3-sonnet-20240229"

/**
 ANTHROPICMODEL_Haiku
 
 A predefined model for generating responses in the form of haikus.
 */

#define ANTHROPICMODEL_Haiku   @"claude-3-haiku-20240307"

#endif
