#ifndef CKCoreFloatLabelTextField_h
#define CKCoreFloatLabelTextField_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

//-------------------------------------------------------------------------------------------------------------------------
// ENUMS
//-------------------------------------------------------------------------------------------------------------------------
#ifndef CKCoreFloatLabelAnimationType
typedef NS_ENUM(NSUInteger, CKCoreFloatLabelAnimationType)
  {
  CKCoreFloatLabelAnimationTypeShow = 0,
  CKCoreFloatLabelAnimationTypeHide
  };
#define CKCoreFloatLabelAnimationType CKCoreFloatLabelAnimationType
#endif

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreFloatLabelTextField : UITextField

/**
 A UILabel that @a floats above the contents of the UITextField
 */
@property (nonatomic, strong) UILabel *floatLabel;

/**
 * The font for @c floatLabel.
 *
 * Defaults to Helvetica Neue Bold 12.0f.
 */
@property (nonatomic, strong) UIFont *floatLabelFont UI_APPEARANCE_SELECTOR;

/**
 * The inactive color for the floatLabel. 
 *
 * Defaults to @c lightGrayColor.
 */
@property (nonatomic, strong) UIColor *floatLabelPassiveColor UI_APPEARANCE_SELECTOR;

/**
 The inactive color for the floatLabel. 
 * 
 * Defaults to @c blueColor.
 */
@property (nonatomic, strong) UIColor *floatLabelActiveColor UI_APPEARANCE_SELECTOR;

/**
 * The animation duration when animating-in the float label.
 * This @c NSNumber value is converted to a @c float.
 *
 * Defaults to 0.25 seconds.
 */
@property (nonatomic, strong) NSNumber *floatLabelShowAnimationDuration UI_APPEARANCE_SELECTOR;

/**
 * The animation duration when animating-out the float label.
 * This @c NSNumber value is converted to a @c float.
 *
 * Defaults to 0.05 seconds.
 */
@property (nonatomic, strong) NSNumber *floatLabelHideAnimationDuration UI_APPEARANCE_SELECTOR;

/**
 * Dismiss the keyboard when interacting with the textField clear button (found on one edge of the textField).
 * This @c NSNumber value is converted to a @c BOOL.
 *
 * Defaults to NO.
 */
@property (nonatomic, assign) NSNumber *dismissKeyboardWhenClearingTextField UI_APPEARANCE_SELECTOR;

/**
 * THe amount of padding to add to the left and right boundaries of the text field.
 *
 * Defaults to 5.0.
 */
@property (nonatomic, assign) CGFloat horizontalPadding UI_APPEARANCE_SELECTOR;

/**
 * Disables the option to @a paste in the @c UIMenuController. 
 * This @c NSNumber value is converted to a @c BOOL.
 *
 * Defaults to YES.
 */
@property (nonatomic, assign) NSNumber *pastingEnabled UI_APPEARANCE_SELECTOR;

/**
 * Disables the option to @a copy in the @c UIMenuController.
 * This @c NSNumber value is converted to a @c BOOL.
 *
 * Defaults to YES.
 */
@property (nonatomic, assign) NSNumber *copyingEnabled UI_APPEARANCE_SELECTOR;

/**
 * Disables the option to @a cut in the @c UIMenuController.
 * This @c NSNumber value is converted to a @c BOOL.
 *
 * Defaults to YES.
 */
@property (nonatomic, assign) NSNumber *cuttingEnabled UI_APPEARANCE_SELECTOR;

/**
 * Disables the option to @a select in the @c UIMenuController.
 * This @c NSNumber value is converted to a @c BOOL.
 *
 * Defaults to YES.
 */
@property (nonatomic, assign) NSNumber *selectEnabled UI_APPEARANCE_SELECTOR;

/**
 * Disables the option to @a select-all in the @c UIMenuController.
 * This @c NSNumber value is converted to a @c BOOL.
 *
 * Defaults to YES.
 */
@property (nonatomic, assign) NSNumber *selectAllEnabled UI_APPEARANCE_SELECTOR;

/**
 * The textField's clear button.
 *
 * Read-only variable.
 */
@property (nonatomic, strong, readonly) UIButton *textFieldClearButton;

/**
 Toggles the float label using an animation
 @param animationType The desired animation (and final state) for the float label.
 */
- (void) toggleFloatLabel:(CKCoreFloatLabelAnimationType) animationType;

@end

#endif

