#ifndef CKCoreGyroscope_h
#define CKCoreGyroscope_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <CoreMotion/CoreMotion.h>

//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreGyroData: NSObject

@property(nonatomic) CMRotationRate rotationRate;

@end

//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreMagnetometerData: NSObject

@property(nonatomic) CMMagneticField magneticField;

@end

//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreAccelerometerData: NSObject

@property (nonatomic) CMAcceleration acceleration;

@end

//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreDeviceMotionData: NSObject

@property (nonatomic) CMDeviceMotion *motion;

@end

//-------------------------------------------------------------------------------------------------------------------------
// TYPEDEFS
//-------------------------------------------------------------------------------------------------------------------------
typedef void (^CKCoreGyroscopeChangedCallback) (CKCoreGyroData *newData, NSError *error);
typedef void (^CKCoreAccelerometerChangedCallback) (CKCoreAccelerometerData *newData, NSError *error);
typedef void (^CKCoreMagnetometerChangedCallback) (CKCoreMagnetometerData *newData, NSError *error);
typedef void (^CKCoreDeviceMotionChangedCallback) (CKCoreDeviceMotionData *newData, NSError *error);

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreGyroscope: NSObject

@property (nonatomic, readonly) Boolean                     isDeviceMotionSensorAvailable;
@property (nonatomic, readonly) Boolean                     isAccelerometerSensorAvailable;
@property (nonatomic, readonly) Boolean                     isGyroscopeSensorAvailable;
@property (nonatomic, readonly) Boolean                     isMagnetometerSensorAvailable;
@property (nonatomic, readonly) CGFloat                     updateInterval;

- (void)                        cleanUp;
- (void)                        startGyroscopeScan:(CKCoreGyroscopeChangedCallback) completionBlock;
- (void)                        startAccelerometerScan:(CKCoreAccelerometerChangedCallback) completionBlock;
- (void)                        startMagnetometerScan:(CKCoreMagnetometerChangedCallback) completionBlock;
- (void)                        startDeviceMotionScan:(CKCoreDeviceMotionChangedCallback) completionBlock;
- (void)                        stopGyroscopeScan;
- (void)                        stopAccelerometerScan;
- (void)                        stopMagnetometerScan;
- (void)                        stopDeviceMotionScan;

@end

#endif 
