#ifndef CKCoreImageCarousel_h
#define CKCoreImageCarousel_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <Availability.h>
#import <QuartzCore/QuartzCore.h>

typedef NS_ENUM(NSInteger, CKCoreImageCarouselType)
{
    CKCoreImageCarouselTypeLinear = 0,
    CKCoreImageCarouselTypeRotary,
    CKCoreImageCarouselTypeInvertedRotary,
    CKCoreImageCarouselTypeCylinder,
    CKCoreImageCarouselTypeInvertedCylinder,
    CKCoreImageCarouselTypeWheel,
    CKCoreImageCarouselTypeInvertedWheel,
    CKCoreImageCarouselTypeCoverFlow,
    CKCoreImageCarouselTypeCoverFlow2,
    CKCoreImageCarouselTypeTimeMachine,
    CKCoreImageCarouselTypeInvertedTimeMachine,
    CKCoreImageCarouselTypeCustom
};


typedef NS_ENUM(NSInteger, CKCoreImageCarouselOption)
{
    CKCoreImageCarouselOptionWrap = 0,
    CKCoreImageCarouselOptionShowBackfaces,
    CKCoreImageCarouselOptionOffsetMultiplier,
    CKCoreImageCarouselOptionVisibleItems,
    CKCoreImageCarouselOptionCount,
    CKCoreImageCarouselOptionArc,
    CKCoreImageCarouselOptionAngle,
    CKCoreImageCarouselOptionRadius,
    CKCoreImageCarouselOptionTilt,
    CKCoreImageCarouselOptionSpacing,
    CKCoreImageCarouselOptionFadeMin,
    CKCoreImageCarouselOptionFadeMax,
    CKCoreImageCarouselOptionFadeRange,
    CKCoreImageCarouselOptionFadeMinAlpha
};


NS_ASSUME_NONNULL_BEGIN

@protocol CKCoreImageCarouselDataSource, CKCoreImageCarouselDelegate;

@interface CKCoreImageCarousel : UIView

@property (nonatomic, weak) IBOutlet __nullable id<CKCoreImageCarouselDataSource> dataSource;
@property (nonatomic, weak) IBOutlet __nullable id<CKCoreImageCarouselDelegate> delegate;
@property (nonatomic, assign) CKCoreImageCarouselType type;
@property (nonatomic, assign) CGFloat perspective;
@property (nonatomic, assign) CGFloat decelerationRate;
@property (nonatomic, assign) CGFloat scrollSpeed;
@property (nonatomic, assign) CGFloat bounceDistance;
@property (nonatomic, assign, getter = isScrollEnabled) BOOL scrollEnabled;
@property (nonatomic, assign, getter = isPagingEnabled) BOOL pagingEnabled;
@property (nonatomic, assign, getter = isVertical) BOOL vertical;
@property (nonatomic, readonly, getter = isWrapEnabled) BOOL wrapEnabled;
@property (nonatomic, assign) BOOL bounces;
@property (nonatomic, assign) CGFloat scrollOffset;
@property (nonatomic, readonly) CGFloat offsetMultiplier;
@property (nonatomic, assign) CGSize contentOffset;
@property (nonatomic, assign) CGSize viewpointOffset;
@property (nonatomic, readonly) NSInteger numberOfItems;
@property (nonatomic, readonly) NSInteger numberOfPlaceholders;
@property (nonatomic, assign) NSInteger currentItemIndex;
@property (nonatomic, strong, readonly) UIView * __nullable currentItemView;
@property (nonatomic, strong, readonly) NSArray *indexesForVisibleItems;
@property (nonatomic, readonly) NSInteger numberOfVisibleItems;
@property (nonatomic, strong, readonly) NSArray *visibleItemViews;
@property (nonatomic, readonly) CGFloat itemWidth;
@property (nonatomic, strong, readonly) UIView *contentView;
@property (nonatomic, readonly) CGFloat toggle;
@property (nonatomic, assign) CGFloat autoscroll;
@property (nonatomic, assign) BOOL stopAtItemBoundary;
@property (nonatomic, assign) BOOL scrollToItemBoundary;
@property (nonatomic, assign) BOOL ignorePerpendicularSwipes;
@property (nonatomic, assign) BOOL centerItemWhenSelected;
@property (nonatomic, readonly, getter = isDragging) BOOL dragging;
@property (nonatomic, readonly, getter = isDecelerating) BOOL decelerating;
@property (nonatomic, readonly, getter = isScrolling) BOOL scrolling;

- (void)scrollByOffset:(CGFloat)offset duration:(NSTimeInterval)duration;
- (void)scrollToOffset:(CGFloat)offset duration:(NSTimeInterval)duration;
- (void)scrollByNumberOfItems:(NSInteger)itemCount duration:(NSTimeInterval)duration;
- (void)scrollToItemAtIndex:(NSInteger)index duration:(NSTimeInterval)duration;
- (void)scrollToItemAtIndex:(NSInteger)index animated:(BOOL)animated;

- (nullable UIView *)itemViewAtIndex:(NSInteger)index;
- (NSInteger)indexOfItemView:(UIView *)view;
- (NSInteger)indexOfItemViewOrSubview:(UIView *)view;
- (CGFloat)offsetForItemAtIndex:(NSInteger)index;
- (nullable UIView *)itemViewAtPoint:(CGPoint)point;

- (void)removeItemAtIndex:(NSInteger)index animated:(BOOL)animated;
- (void)insertItemAtIndex:(NSInteger)index animated:(BOOL)animated;
- (void)reloadItemAtIndex:(NSInteger)index animated:(BOOL)animated;

- (void)reloadData;

@end


@protocol CKCoreImageCarouselDataSource <NSObject>

- (NSInteger) numberOfItemsInCarousel:(CKCoreImageCarousel *)carousel;
- (UIView *)  carousel:(CKCoreImageCarousel *)carousel viewForItemAtIndex:(NSInteger)index reusingView:(nullable UIView *)view;

@optional

- (NSInteger) numberOfPlaceholdersInCarousel:(CKCoreImageCarousel *)carousel;
- (UIView *)  carousel:(CKCoreImageCarousel *)carousel placeholderViewAtIndex:(NSInteger)index reusingView:(nullable UIView *)view;

@end


@protocol CKCoreImageCarouselDelegate <NSObject>
@optional

- (void) carouselWillBeginScrollingAnimation:(CKCoreImageCarousel *) carousel;
- (void) carouselDidEndScrollingAnimation:(CKCoreImageCarousel *) carousel;
- (void) carouselDidScroll:(CKCoreImageCarousel *) carousel;
- (void) carouselCurrentItemIndexDidChange:(CKCoreImageCarousel *) carousel;
- (void) carouselWillBeginDragging:(CKCoreImageCarousel *) carousel;
- (void) carouselDidEndDragging:(CKCoreImageCarousel *) carousel willDecelerate:(BOOL) decelerate;
- (void) carouselWillBeginDecelerating:(CKCoreImageCarousel *) carousel;
- (void) carouselDidEndDecelerating:(CKCoreImageCarousel *) carousel;

- (BOOL)carousel:(CKCoreImageCarousel *)carousel shouldSelectItemAtIndex:(NSInteger)index;
- (void)carousel:(CKCoreImageCarousel *)carousel didSelectItemAtIndex:(NSInteger)index;

- (CGFloat)carouselItemWidth:(CKCoreImageCarousel *)carousel;
- (CATransform3D)carousel:(CKCoreImageCarousel *)carousel itemTransformForOffset:(CGFloat)offset baseTransform:(CATransform3D)transform;
- (CGFloat)carousel:(CKCoreImageCarousel *)carousel valueForOption:(CKCoreImageCarouselOption)option withDefault:(CGFloat)value;

@end

NS_ASSUME_NONNULL_END

#endif

