#ifndef CKCoreNSDictionary_h
#define CKCoreNSDictionary_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNSDictionary

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSDictionary (JSONExtensions)

+ (id) dictionaryWithJSONData:(NSData *) inData error:(NSError **) outError;
+ (id) dictionaryWithJSONString:(NSString *) inJSON error:(NSError **) outError;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSDictionary (Find)

- (id)      getObjectForSubkey:(NSString *) firstKey, ...;
- (Boolean) isKeyExist:(NSString *) withKey;
- (Boolean) isKeyPathExist:(NSString *) withKey;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSDictionary (Save)

- (Boolean)        archiveToFileUrl:(NSURL *) toUrl;
- (Boolean)        archiveToFilePath:(NSString *) toPath;
+ (NSDictionary *) loadFromArchiveFilePath:(NSString *) fromPath;
+ (NSDictionary *) loadFromArchiveFileUrl:(NSURL *) fromUrl;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSDictionary (Values)

- (Boolean)    isNull;

- (NSColor *) colorForKeyname:(NSString *) withKey withDefault:(NSColor *) withDefault;
- (NSColor *) colorForKeyname:(NSString *) withKey;

- (Boolean)    boolForKeyname:(NSString *) withKey withDefault:(Boolean) withDefault;
- (Boolean)    boolForKeyname:(NSString *) withKey;

- (NSInteger)  integerForKeyname:(NSString *) withKey;
- (NSInteger)  integerForKeyname:(NSString *) withKey withDefault:(NSInteger) withDefault;

- (CGFloat)    floatForKeyname:(NSString *) withKey;
- (CGFloat)    floatForKeyname:(NSString *) withKey withDefault:(CGFloat) withDefault;

- (NSUInteger) longForKeyname:(NSString *) withKey;
- (NSUInteger) longForKeyname:(NSString *) withKey withDefault:(NSUInteger) withDefault;

- (NSString *) stringForKeyname:(NSString *) withKey;
- (NSString *) stringForKeyname:(NSString *) withKey withDefault:(NSString *) withDefault;

- (NSRange)    rangeForKeyname:(NSString *) withKey;
- (NSRange)    rangeForKeyname:(NSString *) withKey withDefault:(NSRange) withDefault;

- (NSArray *)  arrayForKeyname:(NSString *) withKey;
- (NSArray *)  arrayForKeyname:(NSString *) withKey withDefault:(NSArray *) withDefault;

- (double)     doubleForKeyname:(NSString *) withKey;
- (double)     doubleForKeyname:(NSString *) withKey withDefault:(double) withDefault;

- (CGSize)     CGSizeForKeyname:(NSString *) withKey;
- (CGSize)     CGSizeForKeyname:(NSString *) withKey withDefault:(CGSize) withDefault;

- (CGPoint)    CGPointForKeyname:(NSString *) withKey;
- (CGPoint)    CGPointForKeyname:(NSString *) withKey withDefault:(CGPoint) withDefault;

- (CGRect)     CGRectForKeyname:(NSString *) withKey;
- (CGRect)     CGRectForKeyname:(NSString *) withKey withDefault:(CGRect) withDefault;

- (NSDate *)   dateForKeyname:(NSString *) withKey withDefault:(NSDate *) withDefault;
- (NSDate *)   dateForKeyname:(NSString *) withKey;

@end

#endif

