
/**
 CKCoreNSOperationQueue.h
 A custom NSOperationQueue category that adds a completion block property to NSOperationQueue
 
 This category allows for the addition of a completion block to be executed once all operations in the queue have finished.
 
 Example Usage:
 
 NSOperationQueue *queue = [[NSOperationQueue alloc] init];
 queue.completionBlock = ^{
    NSLog(@"I'm done!");
 };
 [queue addOperationWithBlock:^{
    NSLog(@"I am an operation.");
 }];
 
 */

#ifndef CKCoreNSOperationQueue_h
#define CKCoreNSOperationQueue_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
#include <CodKits_macOS/CodKits_macOS.h>
#else
#include <CodKits_iOS/CodKits_iOS.h>
#endif

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNSOperationQueue

@end

/**
 A block object to be executed upon completion of all operations in the queue.
 */
typedef void (^NSOperationQueueCompletionBlock)(void);

@interface NSOperationQueue (CompletionBlock)

/**
 The completion block to be executed upon completion of all operations in the queue.
 */
@property (nonatomic, strong, nullable) NSOperationQueueCompletionBlock completionBlock;

@end

#endif

