#ifndef CKCoreNetwork_h
#define CKCoreNetwork_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <SystemConfiguration/SystemConfiguration.h>
#import <netinet/in.h>
#import <CoreTelephony/CTTelephonyNetworkInfo.h>

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSInteger, CKCoreNetworkStatus)
  {
  CKCoreNetworkNotReachable = 0,
  CKCoreNetworkReachableViaWiFi,
  CKCoreNetworkReachableViaWWAN
  };

typedef NS_ENUM(NSInteger, CKCoreNetworkConnectivityType)
  {
  CKCoreNetworkUnknown = 0,
  CKCoreNetworkGPRS,
  CKCoreNetworkEdge,
  CKCoreNetworkWCDMA,
  CKCoreNetworkHSDPA,
  CKCoreNetworkHSUPA,
  CKCoreNetworkCDMA1x,
  CKCoreNetworkCDMAEVDORev0,
  CKCoreNetworkCDMAEVDORevA,
  CKCoreNetworkCDMAEVDORevB,
  CKCoreNetworkeHRPD,
  CKCoreNetworkLTE
  };
  
//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNetworkWatch: NSObject

#define CKCoreNetworkStatusChanged                    @"CKCoreNetworkStatusChanged"

//----------------------------------------------------------------------------------------------------
@property (nonatomic, readwrite) CKCoreNetworkStatus           netStatus;
@property (nonatomic, readwrite) CKCoreNetworkConnectivityType netType;
@property (nonatomic, readwrite) Boolean                       isNetworkConnected;

//----------------------------------------------------------------------------------------------------
+ (CKCoreNetworkWatch *_Nullable) sharedInstance;
+ (void)                          isUrlReachable:(NSURL *) withUrl completionBlock:(void (^)(Boolean isSuccess, NSError *error)) completionBlock;

- (void)                          setAPPBecameActive;
- (void)                          setAPPResignedActive;

@end

//-------------------------------------------------------------------------------------------------------------------------
// 3G         DN:0.5Mbps  UP:0.05Mbps
// 4G 3 bars  DN:1.24Mbps UP:0.14Mbps
// LTE 2 Bars DN:1.44Mbps UP:1.49Mbps
// WIFI       DN:60Mbps   UP:24Mbps
//-------------------------------------------------------------------------------------------------------------------------
extern NSString * _Nullable kReachabilityChangedNotification;

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNetworkInfo: NSObject

//-------------------------------------------------------------------------------------------------------------------------
+ (instancetype _Nullable) reachabilityWithHostName:(NSString * _Nullable) hostName;
+ (instancetype _Nullable) reachabilityWithAddress:(const struct sockaddr * _Nullable) hostAddress;
+ (instancetype _Nullable) reachabilityForInternetConnection;

- (BOOL)                                startNotifier;
- (void)                                stopNotifier;
- (CKCoreNetworkStatus)                 currentReachabilityStatus;
- (BOOL)                                connectionRequired;
- (CKCoreNetworkConnectivityType)       networkWWANType;

@end

NS_ASSUME_NONNULL_END

#endif
