#ifndef CKCoreSystemInfo_h
#define CKCoreSystemInfo_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #import <CodKits_macOS/CodKits_macOS.h>
#else
  #import <CodKits_iOS/CodKits_iOS.h>
#endif

#import <MediaPlayer/MediaPlayer.h>
#import <StoreKit/StoreKit.h>

//----------------------------------------------------------------------------------------------------
typedef NS_ENUM(NSInteger, CKCoreDeviceFamily)
  {
  CKCoreDeviceFamilyUnknown   = 0,
  CKCoreDeviceFamilyiPhone    = 1,
  CKCoreDeviceFamilyiPad      = 2,
  CKCoreDeviceFamilyiPod      = 3,
  CKCoreDeviceFamilySimulator = 4,
  };

typedef NS_ENUM(NSInteger, CKCoreDeviceModel)
  {
  CKCoreDeviceModelUnknown = 0,
  CKCoreDeviceModelSimulatoriPhone,
  CKCoreDeviceModelSimulatoriPad,
  CKCoreDeviceModeliPhone1,
  CKCoreDeviceModeliPhone3G,
  CKCoreDeviceModeliPhone3GS,
  CKCoreDeviceModeliPhone4,
  CKCoreDeviceModeliPhone4S,
  CKCoreDeviceModeliPhone5,
  CKCoreDeviceModeliPhone5c,
  CKCoreDeviceModeliPhone5s,
  CKCoreDeviceModeliPhoneSE,
  CKCoreDeviceModeliPhoneSE2,
  CKCoreDeviceModeliPhone6,
  CKCoreDeviceModeliPhone6Plus,
  CKCoreDeviceModeliPhone6s,
  CKCoreDeviceModeliPhone6sPlus,
  CKCoreDeviceModeliPhone7,
  CKCoreDeviceModeliPhone7Plus,
  CKCoreDeviceModeliPhone8,
  CKCoreDeviceModeliPhone8Plus,
  CKCoreDeviceModeliPhoneX,
  CKCoreDeviceModeliPhoneXR,
  CKCoreDeviceModeliPhoneXS,
  CKCoreDeviceModeliPhoneXSMax,
  CKCoreDeviceModeliPhone11,
  CKCoreDeviceModeliPhone11Pro,
  CKCoreDeviceModeliPhone11ProMax,
  CKCoreDeviceModeliPhone12Mini,
  CKCoreDeviceModeliPhone12,
  CKCoreDeviceModeliPhone12Pro,
  CKCoreDeviceModeliPhone12ProMax,
  CKCoreDeviceModeliPhone13Mini,
  CKCoreDeviceModeliPhone13,
  CKCoreDeviceModeliPhone13Pro,
  CKCoreDeviceModeliPhone13ProMax,
  CKCoreDeviceModeliPhone14,
  CKCoreDeviceModeliPhone14Plus,
  CKCoreDeviceModeliPhone14Pro,
  CKCoreDeviceModeliPhone14ProMax,
  CKCoreDeviceModeliPhone15,
  CKCoreDeviceModeliPhone15Plus,
  CKCoreDeviceModeliPhone15Pro,
  CKCoreDeviceModeliPhone15ProMax,
  CKCoreDeviceModeliPhone16,
  CKCoreDeviceModeliPhone16Plus,
  CKCoreDeviceModeliPhone16Pro,
  CKCoreDeviceModeliPhone16ProMax,
  CKCoreDeviceModeliPad1,
  CKCoreDeviceModeliPad2,
  CKCoreDeviceModeliPad3,
  CKCoreDeviceModeliPad4,
  CKCoreDeviceModeliPadMini1,
  CKCoreDeviceModeliPadMini2,
  CKCoreDeviceModeliPadMini3,
  CKCoreDeviceModeliPadMini4,
  CKCoreDeviceModeliPadMini5,
  CKCoreDeviceModeliPadMini6,
  CKCoreDeviceModeliPadAir1,
  CKCoreDeviceModeliPadAir2,
  CKCoreDeviceModeliPadAir3,
  CKCoreDeviceModeliPadAir4,
  CKCoreDeviceModeliPadPro9p7Inch,
  CKCoreDeviceModeliPadPro10p5Inch,
  CKCoreDeviceModeliPadPro12p9Inch,
  CKCoreDeviceModeliPadPro12p9Inch2,
  CKCoreDeviceModeliPadPro11p,
  CKCoreDeviceModeliPadPro11p1TB,
  CKCoreDeviceModeliPadPro11pInch4,
  CKCoreDeviceModeliPadPro12p9Inch3,
  CKCoreDeviceModeliPadPro12p9Inch31TB,
  CKCoreDeviceModeliPadPro12p9Inch4,
  CKCoreDeviceModeliPadPro12p9Inch6,
  CKCoreDeviceModeliPad5,
  CKCoreDeviceModeliPad6,
  CKCoreDeviceModeliPad7,
  CKCoreDeviceModeliPad8,
  CKCoreDeviceModeliPad9,
  CKCoreDeviceModeliPod1,
  CKCoreDeviceModeliPod2,
  CKCoreDeviceModeliPod3,
  CKCoreDeviceModeliPod4,
  CKCoreDeviceModeliPod5,
  CKCoreDeviceModeliPod6,
  CKCoreDeviceModeliPod7
  };

typedef NS_ENUM(NSInteger, CKCoreDeviceDisplay)
  {
  CKCoreDeviceDisplayUnknown  = 0,
  CKCoreDeviceDisplay3p5Inch,
  CKCoreDeviceDisplay4Inch,
  CKCoreDeviceDisplay4p7Inch,
  CKCoreDeviceDisplay5p4Inch,
  CKCoreDeviceDisplay5p5Inch,
  CKCoreDeviceDisplay5p8Inch,
  CKCoreDeviceDisplay6p1Inch,
  CKCoreDeviceDisplay6p3Inch,
  CKCoreDeviceDisplay6p5Inch,
  CKCoreDeviceDisplay6p7Inch,
  CKCoreDeviceDisplay6p9Inch,
  CKCoreDeviceDisplay7p9Inch,
  CKCoreDeviceDisplay9p7Inch,
  CKCoreDeviceDisplay10p2Inch,
  CKCoreDeviceDisplay10p5Inch,
  CKCoreDeviceDisplay10p9Inch,
  CKCoreDeviceDisplay11pInch,
  CKCoreDeviceDisplay12p9Inch,
  CKCoreDeviceDisplay13pInch,
  };

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemInfo: NSObject

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemInfo (Device)

+ (NSString *)     deviceSerialNumber;
+ (NSString *)     devicePreferredLanguage;
+ (NSString *)     deviceCountryCode;
+ (NSString *)     deviceCountryName;
+ (NSString *)     deviceUUID;
+ (NSDictionary *) deviceModel;
+ (NSDictionary *) deviceModelFor:(CKCoreDeviceModel) withModel;
+ (Boolean)        deviceIsInPortraitMode;
+ (Boolean)        deviceIsInLandscapeMode;
+ (Boolean)        deviceIsInFlatMode;
+ (Boolean)        deviceIsInverted;
+ (NSUInteger)     numberOfProcessors;
+ (NSUInteger)     numberOfActiveProcessors;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemInfo (APPVersion)

+ (NSString *) appVersionShort;
+ (NSString *) appVersion;
+ (NSString *) appVersionBuild;
+ (NSString *) appVersionLong;
+ (NSString *) appBundleName;
+ (NSString *) appBundleIdentifier;
+ (void)       appStoreInfo:(void (^)(NSDictionary * dictionary)) completionHandler;
+ (void)       isAppStoreUpdateAvailable:(void (^)(Boolean updateAvailable, NSString *newVersion, NSDictionary *dictionary)) completionHandler;
+ (Boolean)    isAPPRunningInXCode;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemInfo (APPRun)

+ (Boolean) isDebuggerAttached;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemInfo (Screen)

+ (CGFloat)                screenWidth;
+ (CGFloat)                screenRealWidth;
+ (CGFloat)                screenHeight;
+ (CGFloat)                screenRealHeight;
+ (CGFloat)                screenRatio;
+ (CGSize)                 screenSize;
+ (CGSize)                 screenRealSize;
+ (CGRect)                 screenBounds;
+ (Boolean)                screenHasTopNotch;
+ (Boolean)                screenHasBottomNotch;
+ (UIEdgeInsets)           screenSafeArea;
+ (CGFloat)                screenScale;
+ (CGFloat)                screenScaleForModel:(CKCoreDeviceModel) withModel;
+ (CGFloat)                screenPPi;
+ (CGFloat)                screenPPiForModel:(CKCoreDeviceModel) withModel;
+ (CGFloat)                screenHorizontalInches;
+ (CGFloat)                screenHorizontalInchesForModel:(CKCoreDeviceModel) withModel;
+ (CGFloat)                screenVerticalInches;
+ (CGFloat)                screenVerticalInchesForModel:(CKCoreDeviceModel) withModel;
+ (CGFloat)                screenDiagonalInches;
+ (CGFloat)                screenDiagonalInchesForModel:(CKCoreDeviceModel) withModel;
+ (NSDictionary *)         screenDimensions;
+ (CGFloat)                screenTopNotchHeight;
+ (CGFloat)                screenBottomNotchHeight;
+ (NSWindow *)             screenKeyWindow;
+ (NSArray <NSWindow *> *) screenWindows;

@end

#endif 
