#ifndef CKCoreTorch_h
#define CKCoreTorch_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
/**
 * @interface CKCoreTorch
 * @brief This class provides methods to control the torch (flashlight) on a device.
 *
 * CKCoreTorch is an NSObject subclass that allows setting the torch level on supported devices.
 */
@interface CKCoreTorch: NSObject

/**
 * @brief Sets the torch (flashlight) to the specified level.
 *
 * This method sets the brightness level of the device's torch.
 *
 * @param TorchLevel The desired brightness level of the torch. This should be a float value where 0.0 represents
 *        the torch being off and 1.0 represents the maximum brightness.
 *
 * @discussion
 * Use this method to adjust the brightness of the torch on a supported device. Ensure that the device
 * has a torch before calling this method.
 *
 * Example usage:
 * @code
 * [CKCoreTorch setTorchToLevel:0.5];
 * @endcode
 */
+ (void) setTorchToLevel:(float) TorchLevel;

@end

#endif 
