#ifndef CKCoreDropdownAlert_h
#define CKCoreDropdownAlert_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <QuartzCore/QuartzCore.h>

//-------------------------------------------------------------------------------------------------------------------------
// PROTOCOL
//-------------------------------------------------------------------------------------------------------------------------
@protocol CKCoreDropdownAlertDelegate <NSObject>

- (void) dropdownNotificationTopButtonTapped;
- (void) dropdownNotificationBottomButtonTapped;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreDropdownAlert: NSObject

typedef NS_ENUM(NSInteger, CKCoreDropdownAlertEvent)
  {
  CKCoreDropdownAlertEventTopButton,
  CKCoreDropdownAlertEventBottomButton,
  CKCoreDropdownAlertEventTap,
  CKCoreDropdownAlertEventDelay,
  };

typedef void (^CKCoreDropdownAlertBlock)(CKCoreDropdownAlertEvent event);

@property (nonatomic, strong) id<CKCoreDropdownAlertDelegate> notificationDelegate;
@property (nonatomic, strong) UIView                                *notificationView;
@property (nonatomic, strong) NSString                              *titleText;
@property (nonatomic, strong) NSString                              *subtitleText;
@property (nonatomic, strong) UIImage                               *image;
@property (nonatomic, strong) UIColor                               *colorBackground;
@property (nonatomic, strong) UIColor                               *colorText;
@property (nonatomic, strong) NSString                              *topButtonText;
@property (nonatomic, strong) NSString                              *bottomButtonText;
@property (nonatomic) BOOL                                           isBeingShown;
@property (nonatomic) BOOL                                           isImageRounded;
@property (nonatomic) BOOL                                           dismissOnTap;
@property (nonatomic) NSTimeInterval                                 dismissAfterDelay;
 
- (void) listenEventsWithBlock:(CKCoreDropdownAlertBlock) block;
- (void) presentInView:(UIView *) view withGravityAnimation:(BOOL) animation;
- (void) dismissWithGravityAnimation:(BOOL) animation;

@end

#endif

