#ifndef CKCoreFileSystem_h
#define CKCoreFileSystem_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <ImageIO/ImageIO.h>

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreFileSystem : NSObject

+ (id)                         attributeOfItemAtPath:(NSString *) path forKey:(NSString *) key;
+ (id)                         attributeOfItemAtUrl:(NSURL *) url forKey:(NSString *) key;
+ (id)                         attributeOfItemAtPath:(NSString *) path forKey:(NSString *) key error:(NSError **) error;
+ (id)                         attributeOfItemAtUrl:(NSURL *) url forKey:(NSString *) key error:(NSError **) error;
+ (NSDictionary *)             attributesOfItemAtPath:(NSString *) path error:(NSError **) error;
+ (NSDictionary *)             attributesOfItemAtUrl:(NSURL *) url error:(NSError **) error;
+ (NSDictionary *)             attributesOfItemAtPath:(NSString *) path;
+ (NSDictionary *)             attributesOfItemAtUrl:(NSURL *) url;

+ (Boolean)                    copyItemAtPath:(NSString *) path toPath:(NSString *)toPath;
+ (Boolean)                    copyItemAtUrl:(NSURL *) Url toUrl:(NSURL *) toUrl;

+ (Boolean)                    copyItemAtPath:(NSString *) path toPath:(NSString *)toPath error:(NSError **)error;
+ (Boolean)                    copyItemAtUrl:(NSURL *) url toUrl:(NSURL *) toUrl error:(NSError **) error;

+ (Boolean)                    copyItemAtPath:(NSString *) path toPath:(NSString *) toPath overwrite:(BOOL) overwrite;
+ (Boolean)                    copyItemAtUrl:(NSURL *) url toPath:(NSURL *) toUrl overwrite:(BOOL) overwrite;

+ (Boolean)                    copyItemAtPath:(NSString *) path toPath:(NSString *) toPath overwrite:(BOOL) overwrite error:(NSError **)error;
+ (Boolean)                    copyItemAtUrl:(NSURL *) url toUrl:(NSURL *) toUrl overwrite:(BOOL) overwrite error:(NSError **) error;

+ (Boolean)                    copyItemsAtUrl:(NSArray <NSURL *> *) Urls toDirectory:(NSURL *) toDirectory;
+ (Boolean)                    copyItemsAtPath:(NSArray <NSString *> *) Paths toDirectory:(NSURL *) toDirectory;

+ (Boolean)                    copyItemAtPath:(NSString *) Path toDirectory:(NSURL *) toDirectory;
+ (Boolean)                    copyItemAtUrl:(NSURL *) Url toDirectory:(NSURL *) toDirectory;

+ (Boolean)                    createDirectoryForFileAtPath:(NSString *) path;
+ (Boolean)                    createDirectoryForFileAtUrl:(NSURL *) url;

+ (Boolean)                    createDirectoryForUrl:(NSURL *) url;
+ (Boolean)                    createDirectoryForPath:(NSString *) path;

+ (Boolean)                    createDirectoryForFileAtPath:(NSString *) path error:(NSError **) error;
+ (Boolean)                    createDirectoryForFileAtUrl:(NSURL *) url error:(NSError **) error;

+ (Boolean)                    createDirectoryForPath:(NSString *) path error:(NSError **) error;
+ (Boolean)                    createHiddenDirectoryForPath:(NSString *) path;

+ (Boolean)                    createFileAtPath:(NSString *) path;
+ (Boolean)                    createFileAtPath:(NSString *) path error:(NSError **) error;

+ (Boolean)                    createFileAtPath:(NSString *) path overwrite:(BOOL) overwrite;
+ (Boolean)                    createFileAtPath:(NSString *) path overwrite:(BOOL) overwrite error:(NSError **) error;

+ (Boolean)                    createFileAtPath:(NSString *) path withContent:(NSObject *) content;
+ (Boolean)                    createFileAtPath:(NSString *) path withContent:(NSObject *) content error:(NSError **) error;

+ (Boolean)                    createFileAtPath:(NSString *) path withContent:(NSObject *) content overwrite:(BOOL) overwrite;
+ (Boolean)                    createFileAtPath:(NSString *) path withContent:(NSObject *) content overwrite:(BOOL) overwrite error:(NSError **)error;

+ (NSDate *)                   creationDateOfItemAtPath:(NSString *)path;
+ (NSDate *)                   creationDateOfItemAtPath:(NSString *)path error:(NSError **)error;

+ (NSDate *)                   modificationDateOfItemAtPath:(NSString *) path;
+ (NSDate *)                   modificationDateOfItemAtPath:(NSString *) path error:(NSError **) error;

+ (Boolean)                    emptyCachesDirectory;
+ (Boolean)                    emptyTemporaryDirectory;

+ (Boolean)                    existsItemAtPath:(NSString *)path;

+ (Boolean)                    isDirectoryItemAtPath:(NSString *) path;
+ (Boolean)                    isDirectoryItemAtPath:(NSString *) path error:(NSError **) error;

+ (Boolean)                    isEmptyItemAtPath:(NSString *) path;
+ (Boolean)                    isEmptyItemAtPath:(NSString *) path error:(NSError **) error;

+ (Boolean)                    isFileItemAtPath:(NSString *) path;
+ (Boolean)                    isFileItemsAtPath:(NSArray<NSString *> *) paths;
+ (Boolean)                    isFileItemAtPath:(NSString *) path error:(NSError **) error;
+ (Boolean)                    isFileItemAtURL:(NSURL *) Url;

+ (Boolean)                    isExecutableItemAtPath:(NSString *) path;
+ (Boolean)                    isReadableItemAtPath:(NSString *) path;
+ (Boolean)                    isWritableItemAtPath:(NSString *) path;

+ (void)                       listDirectories:(NSURL *) atDirectory goDeep:(Boolean) goDeep progressBlock:(void (^)(NSString *itemPath, NSString *itemName, Boolean isPackage)) progressBlock completionBlock:(void (^)(void)) completionBlock;
+ (void)                       listFilesInDirectory:(NSURL *) atDirectory goDeep:(Boolean) goDeep progressBlock:(void (^)(NSString *itemPath, NSString *itemName, Boolean isPackage)) progressBlock completionBlock:(void (^)(void)) completionBlock;
+ (void)                       listFilesOlderThan:(NSURL *) atDirectory goDeep:(Boolean) goDeep olderThan:(NSTimeInterval) olderThan progressBlock:(void (^)(NSString *itemPath, NSString *itemName, Boolean isPackage)) progressBlock completionBlock:(void (^)(void)) completionBlock;

+ (void)                       enumerateFilesInDirectoryAtPath:(NSString *) path deep:(BOOL) deep callback:(void(^)(NSURL *url)) callback;

+ (NSArray *)                  listDirectoriesInDirectoryAtPath:(NSString *) path;
+ (NSArray *)                  listDirectoriesInDirectoryAtPath:(NSString *) path deep:(BOOL) deep;

+ (NSArray *)                  listFilesInDirectoryAtPath:(NSString *) path;
+ (NSInteger)                  countFilesInDirectoryAtPath:(NSString *) path;

+ (NSArray *)                  listFilesInDirectoryAtPath:(NSString *) path deep:(BOOL) deep;
+ (NSInteger)                  countFilesInDirectoryAtPath:(NSString *) path deep:(BOOL) deep;

+ (NSArray *)                  listFilesInDirectoryAtPath:(NSString *) path withExtension:(NSString *) extension;
+ (NSArray *)                  listFilesInDirectoryAtPath:(NSString *) path withExtension:(NSString *) extension deep:(BOOL) deep;
+ (NSArray *)                  listFilesInDirectoryAtPathOlderThan:(NSString *) path withExtension:(NSString *) extension deep:(BOOL) deep olderThan:(NSDate *) olderThan;
+ (NSArray <NSDictionary *> *) listFilesInDirectoryAtPathSortedByDate:(NSString *) path withExtension:(NSString *) extension;

+ (NSArray *)                  listFilesInDirectoryAtPath:(NSString *) path withPrefix:(NSString *) prefix;
+ (NSArray *)                  listFilesInDirectoryAtPath:(NSString *) path withPrefix:(NSString *) prefix deep:(BOOL) deep;

+ (NSArray *)                  listFilesInDirectoryAtPath:(NSString *) path withSuffix:(NSString *) suffix;
+ (NSArray *)                  listFilesInDirectoryAtPath:(NSString *) path withSuffix:(NSString *) suffix deep:(BOOL) deep;

+ (NSArray *)                  listItemsInDirectoryAtPath:(NSString *) path deep:(BOOL) deep;

+ (Boolean)                    moveItemAtPath:(NSString *)path toPath:(NSString *)toPath;
+ (Boolean)                    moveItemAtPath:(NSString *)path toPath:(NSString *)toPath error:(NSError **)error;

+ (Boolean)                    moveItemAtPath:(NSString *)path toPath:(NSString *)toPath overwrite:(BOOL)overwrite;
+ (Boolean)                    moveItemAtPath:(NSString *)path toPath:(NSString *)toPath overwrite:(BOOL)overwrite error:(NSError **)error;

+ (Boolean)                    moveItemAtUrl:(NSURL *) Url toUrl:(NSURL *) toUrl;
+ (Boolean)                    moveItemsAtUrl:(NSArray <NSURL *> *) Urls toDirectory:(NSURL *) toDirectory;
+ (Boolean)                    moveItemAtUrl:(NSURL *) Url toDirectory:(NSURL *) toDirectory;

+ (NSString *)                 pathForDownloadsDirectory;

+ (NSString *)                 pathForDesktopDirectory;

+ (NSString *)                 pathForApplicationSupportDirectory;
+ (NSString *)                 pathForApplicationSupportDirectoryWithPath:(NSString *) path;

+ (NSString *)                 pathForCachesDirectory;
+ (NSString *)                 pathForCachesDirectoryWithPath:(NSString *) path;

+ (NSString *)                 pathForDocumentsDirectory;
+ (NSString *)                 pathForDocumentsDirectoryWithPath:(NSString *) path;

+ (NSString *)                 pathForLibraryDirectory;
+ (NSString *)                 pathForLibraryDirectoryWithPath:(NSString *) path;

+ (NSString *)                 pathForMainBundleDirectory;
+ (NSString *)                 pathForMainBundleDirectoryWithPath:(NSString *) path;

+ (NSString *)                 pathForPlistNamed:(NSString *) name;

+ (NSString *)                 pathForTemporaryDirectory;
+ (NSString *)                 pathForTemporaryDirectoryWithPath:(NSString *) path;

+ (NSString *)                 readFileAtPath:(NSString *) path;
+ (NSString *)                 readFileAtPath:(NSString *) path error:(NSError **) error;

+ (NSArray *)                  readFileAtPathAsArray:(NSString *) path;

+ (NSObject *)                 readFileAtPathAsCustomModel:(NSString *) path;

+ (NSData *)                   readFileAtPathAsData:(NSString *)path;
+ (NSData *)                   readFileAtPathAsData:(NSString *)path error:(NSError **)error;
+ (NSDate *)                   modificationDateForFileAtPath:(NSString *) path;

+ (NSDictionary *)             readFileAtPathAsDictionary:(NSString *)path;

+ (NSImage *)                  readFileAtPathAsImage:(NSString *)path;
+ (NSImage *)                  readFileAtPathAsImage:(NSString *)path error:(NSError **)error;

+ (NSImageView *)              readFileAtPathAsImageView:(NSString *)path;
+ (NSImageView *)              readFileAtPathAsImageView:(NSString *)path error:(NSError **)error;

+ (NSJSONSerialization *)      readFileAtPathAsJSON:(NSString *)path;
+ (NSJSONSerialization *)      readFileAtPathAsJSON:(NSString *)path error:(NSError **)error;

+ (NSMutableArray *)           readFileAtPathAsMutableArray:(NSString *)path;
+ (NSMutableData *)            readFileAtPathAsMutableData:(NSString *)path;
+ (NSMutableData *)            readFileAtPathAsMutableData:(NSString *)path error:(NSError **)error;

+ (NSMutableDictionary *)      readFileAtPathAsMutableDictionary:(NSString *)path;

+ (NSString *)                 readFileAtPathAsString:(NSString *)path;
+ (NSString *)                 readFileAtPathAsString:(NSString *)path error:(NSError **)error;

+ (Boolean)                    removeFilesInDirectoryAtPath:(NSString *)path;
+ (Boolean)                    removeFilesInDirectoryAtPath:(NSString *)path error:(NSError **)error;

+ (Boolean)                    removeFilesInDirectoryAtPath:(NSString *)path withExtension:(NSString *)extension;
+ (Boolean)                    removeFilesInDirectoryAtPath:(NSString *)path withExtension:(NSString *)extension error:(NSError **)error;

+ (Boolean)                    removeFilesInDirectoryAtPath:(NSString *)path withPrefix:(NSString *)prefix;
+ (Boolean)                    removeFilesInDirectoryAtPath:(NSString *)path withPrefix:(NSString *)prefix error:(NSError **)error;

+ (Boolean)                    removeFilesInDirectoryAtPath:(NSString *)path withSuffix:(NSString *)suffix;
+ (Boolean)                    removeFilesInDirectoryAtPath:(NSString *)path withSuffix:(NSString *)suffix error:(NSError **)error;

+ (Boolean)                    removeItemsInDirectoryAtPath:(NSString *)path;
+ (Boolean)                    removeItemsInDirectoryAtPath:(NSString *)path error:(NSError **)error;

+ (Boolean)                    removeItemAtPath:(NSString *)path;
+ (Boolean)                    removeItemAtPath:(NSString *)path error:(NSError **)error;

+ (Boolean)                    removeDirectoryAtPath:(NSString *) path;

+ (Boolean)                    removeDirectoryAtUrl:(NSURL *) url;

+ (Boolean)                    renameItemAtPath:(NSString *)path withName:(NSString *)name;
+ (Boolean)                    renameItemAtPath:(NSString *)path withName:(NSString *)name error:(NSError **)error;
+ (Boolean)                    removeItemsAtPath:(NSArray *) paths error:(NSError **) error;
+ (Boolean)                    removeItemsAtPath:(NSArray *) paths;

+ (Boolean)                    removeItemAtUrl:(NSURL *) url;
+ (Boolean)                    removeItemsAtUrl:(NSArray <NSURL *> *) Urls;
+ (NSArray<NSURL *> *)         removeFilesOlderThan:(NSDate *) earliestDateAllowed inDirectory:(NSURL *) directory;

+ (NSString *)                 sizeFormatted:(NSNumber *) size;

+ (NSString *)                 sizeFormattedOfDirectoryAtPath:(NSString *) path;
+ (NSString *)                 sizeFormattedOfDirectoryAtPath:(NSString *) path error:(NSError **) error;

+ (NSString *)                 sizeFormattedOfFileAtPath:(NSString *) path;
+ (NSString *)                 sizeFormattedOfFileAtPath:(NSString *) path error:(NSError **) error;

+ (NSString *)                 sizeFormattedOfItemAtPath:(NSString *) path;
+ (NSString *)                 sizeFormattedOfItemAtPath:(NSString *) path error:(NSError **) error;

+ (NSNumber *)                 sizeOfDirectoryAtPath:(NSString *) path;
+ (NSNumber *)                 sizeOfDirectoryAtPath:(NSString *) path error:(NSError **) error;

+ (NSNumber *)                 sizeOfFileAtUrl:(NSURL *) url;
+ (NSNumber *)                 sizeOfFileAtPath:(NSString *) path;
+ (NSNumber *)                 sizeOfFileAtPath:(NSString *) path error:(NSError **) error;

+ (NSNumber *)                 sizeOfItemAtPath:(NSString *)path;
+ (NSNumber *)                 sizeOfItemAtPath:(NSString *)path error:(NSError **)error;

+ (NSURL *)                    urlForItemAtPath:(NSString *)path;

+ (Boolean)                    writeFileAtPath:(NSString *) path content:(NSObject *) content;
+ (Boolean)                    writeFileAtPath:(NSString *) path content:(NSObject *) content error:(NSError **) error;

+ (NSDictionary *)             metadataOfImageAtPath:(NSString *)path;
+ (NSDictionary *)             exifDataOfImageAtPath:(NSString *)path;
+ (NSDictionary *)             tiffDataOfImageAtPath:(NSString *)path;

+ (NSDictionary *)             xattrOfItemAtPath:(NSString *)path;
+ (NSString *)                 xattrOfItemAtPath:(NSString *)path getValueForKey:(NSString *)key;
+ (Boolean)                    xattrOfItemAtPath:(NSString *)path hasValueForKey:(NSString *)key;
+ (Boolean)                    xattrOfItemAtPath:(NSString *)path removeValueForKey:(NSString *)key;
+ (Boolean)                    xattrOfItemAtPath:(NSString *)path setValue:(NSString *)value forKey:(NSString *)key;


@end

#endif

