#ifndef CKCoreHeading_h
#define CKCoreHeading_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <CoreMotion/CoreMotion.h>

typedef NS_ENUM(NSInteger, CKCoreHeadingAuthorization)
  {
  CKCoreHeadingAuthorizedAlways      = 0,
  CKCoreHeadingAuthorizedWhenInUse   = 1,
  CKCoreHeadingDenied                = 2,
  CKCoreHeadingRequest               = 3,
  CKCoreHeadingRestricted            = 4,
  };

//-------------------------------------------------------------------------------------------------------------------------
// TYPEDEFS
//-------------------------------------------------------------------------------------------------------------------------
typedef void (^CKCoreHeadingAuthorizationStatusChangedCallback) (CLLocationManager *manager);
typedef void (^CKCoreHeadingChangedCallback) (CLHeading *newHeading, NSError *error);

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreHeading: NSObject
 
- (void)                        onAuthorizationStatusChanged:(CKCoreHeadingAuthorizationStatusChangedCallback) completionBlock;
- (void)                        cleanUp;
- (void)                        requestInUseAuthorization:(CKCoreHeadingAuthorizationStatusChangedCallback) completionBlock;
- (void)                        requestAlwaysAuthorization:(CKCoreHeadingAuthorizationStatusChangedCallback) completionBlock;
- (void)                        requestFullAccuracyAuthorization:(void (^)(Boolean isSuccess, NSError *error)) completionBlock;
- (void)                        requestCurrentHeading:(CKCoreHeadingChangedCallback) completionBlock;
- (void)                        startHeadingScan:(CKCoreHeadingChangedCallback) completionBlock;
- (void)                        stopHeadingScan;

@property (nonatomic, readonly) Boolean                     isSensorAvailable;
@property (nonatomic) CLLocationDegrees                     desiredAccuracy;
@property (nonatomic) CLDeviceOrientation                   deviceOrientation;
@property (nonatomic, readonly) CLHeading                  *lastValidHeading;
@property (nonatomic, readonly) CKCoreHeadingAuthorization  isAuthorized;
@property (nonatomic, readonly) NSString                   *cardinalDirection;

@end

#endif 
