#ifndef CKCoreLocation_h
#define CKCoreLocation_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <CoreMotion/CoreMotion.h>

typedef NS_ENUM(NSInteger, CKCoreLocationAuthorization)
  {
  CKCoreLocationAuthorizedAlways      = 0,
  CKCoreLocationAuthorizedWhenInUse   = 1,
  CKCoreLocationDenied                = 2,
  CKCoreLocationRequest               = 3,
  CKCoreLocationRestricted            = 4,
  };

typedef NS_ENUM(NSInteger, CKCoreLocationFormat)
  {
  CKCoreLocationDegreesMinutesSeconds = 0,
  CKCoreLocationDegreesDecimalMinutes = 1,
  CKCoreLocationDecimalDegrees        = 2,
  };

//-------------------------------------------------------------------------------------------------------------------------
// TYPEDEFS
//-------------------------------------------------------------------------------------------------------------------------
typedef void (^CKCoreLocationAuthorizationStatusChangedCallback) (CLLocationManager *manager);
typedef void (^CKCoreLocationChangedCallback) (CLLocation *newLocation, NSError *error);

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreLocation: NSObject
 
+ (NSString *)                  latitudeToString:(CLLocation *) withLocation withFormat:(CKCoreLocationFormat) withFormat;
+ (NSString *)                  longitudeToString:(CLLocation *) withLocation withFormat:(CKCoreLocationFormat) withFormat;

- (void)                        onAuthorizationStatusChanged:(CKCoreLocationAuthorizationStatusChangedCallback) completionBlock;
- (void)                        cleanUp;
- (void)                        requestInUseAuthorization:(CKCoreLocationAuthorizationStatusChangedCallback) completionBlock;
- (void)                        requestAlwaysAuthorization:(CKCoreLocationAuthorizationStatusChangedCallback) completionBlock;
- (void)                        requestFullAccuracyAuthorization:(void (^)(Boolean isSuccess, NSError *error)) completionBlock;
- (void)                        requestCurrentLocation:(CKCoreLocationChangedCallback) completionBlock;
- (void)                        startLocationScan:(CKCoreLocationChangedCallback) completionBlock;
- (void)                        stopLocationScan;
- (void)                        startBackgroundLocationScan:(CKCoreLocationChangedCallback) completionBlock;
- (void)                        requestCurrentPlacemark:(void (^)(CLPlacemark *placemark)) completionBlock;

@property (nonatomic, readonly) Boolean                     isSensorAvailable;
@property (nonatomic) CLLocationAccuracy                    desiredAccuracy;
@property (nonatomic, readonly) CLLocation                 *lastValidLocation;
@property (nonatomic, readonly) CGFloat                     distanceTraveled;
@property (nonatomic, readonly) Boolean                     isPreciseLocation;
@property (nonatomic, readonly) CKCoreLocationAuthorization isAuthorized;

@end

#endif 
