#ifndef CKCoreNSImage_h
#define CKCoreNSImage_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <ImageIO/ImageIO.h>

NS_ASSUME_NONNULL_BEGIN

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (Extended)

- (NSData *)  PNGRepresentation:(CGFloat) withFactor;
- (NSData *)  JPGRepresentation:(CGFloat) withFactor;
- (Boolean)   saveAsPNG:(NSString *) toFilePath withFactor:(CGFloat) withFactor;
- (Boolean)   saveAsJPG:(NSString *) toFilePath withFactor:(CGFloat) withFactor;

+ (NSImage *) imageWithContentsOfURL:(NSURL *) url;
+ (CGRect)    calculateImageCropRectangle:(NSImage *) withImage withSize:(CGSize) withSize;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (animatedGIF)

+ (UIImage *) animatedImageWithAnimatedGIFData:(NSData *) theData;
+ (UIImage *) animatedImageWithAnimatedGIFURL:(NSURL *) theURL;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (Async)

+ (void) imageFromUrl:(NSURL *) url completionBlock:(void (^)(NSImage *image)) completionBlock;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (Crop)

- (void) cropToRectangle:(CGRect) withCropRect completionBlock:(void (^)(UIImage *croppedImage)) completionBlock;
- (void) cropToCenteredSize:(CGSize) withCropSize completionBlock:(void (^)(UIImage *croppedImage)) completionBlock;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface UIImage (ImageEffects)

- (UIImage *_Nullable) applyLightEffect;
- (UIImage *_Nullable) applyExtraLightEffect;
- (UIImage *_Nullable) applyDarkEffect;
- (UIImage *_Nullable) applyTintEffectWithColor:(UIColor *_Nullable) TintColor;
- (UIImage *_Nullable) applyLightEffect:(CGFloat) BlurRadius;
- (UIImage *_Nullable) applyExtraLightEffect:(CGFloat) BlurRadius;
- (UIImage *_Nullable) applyDarkEffect:(CGFloat) BlurRadius;
- (UIImage *_Nullable) applyBlurWithRadius:(CGFloat) BlurRadius tintColor:(UIColor *_Nullable) TintColor saturationDeltaFactor:(CGFloat) SaturationDeltaFactor maskImage:(UIImage *_Nullable) MaskImage;
- (UIImage *_Nullable) applyAlpha:(CGFloat) alpha;
- (UIImage *_Nullable) colorizeWithColor:(UIColor *_Nullable) color;

+ (UIImage *_Nullable) applyTintWithColor:(UIImage *_Nullable) withImage withColor:(UIColor *_Nullable) withColor;
+ (UIImage *_Nullable) imageWithColor:(UIColor *_Nullable) Color;
+ (UIImage *_Nullable) convertViewToImage;
+ (UIImage *_Nullable) convertViewToImage:(UIView *_Nullable) View;
+ (UIImage *_Nullable) convertViewToImage:(UIView *_Nullable) View WithScale:(CGFloat) Scale;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (CGImage)

- (CGImageRef _Nullable)  CGImage;
+ (NSImage *_Nullable)    imageWithCGImage:(CGImageRef _Nullable ) imageRef;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (CIImage)

- (CIImage *) toCIImage;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (Text)

- (UIImage *) imageWithCenteredText:(NSString *) text withfont:(UIFont *) font;
- (UIImage *) imageWithTopRightText:(NSString *) text withfont:(UIFont *) font withOffset:(CGFloat) withOffset;
- (UIImage *) imageWithText:(NSString *) text atPoint:(CGPoint) point withFont:(UIFont *) font;
- (UIImage *) imageWithText:(NSString *) text atPoint:(CGPoint) point withFont:(UIFont *) font withAlignment:(NSTextAlignment) withAlignment withColor:(UIColor *) withColor;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (FillColor)

+ (NSImage *_Nullable) filledWithColor:(CGSize) withSize withColor:(NSColor *_Nullable) withColor;
- (NSImage *_Nullable) tintWithColor:(NSColor *_Nullable) withColor;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (Scale)

- (NSImage *_Nullable) scaleToPercentSize:(CGFloat) withPercent;
- (NSImage *_Nullable) scaleToFactorSize:(CGFloat) withFactor;
- (NSImage *_Nullable) scaleToSize:(CGSize) withSize;
- (NSImage *_Nullable) scaleToSizeSmooth:(CGSize) newSize;
- (NSImage *_Nullable) scaleToHeight:(CGFloat) withHeight;
- (NSImage *_Nullable) scaleToWidth:(CGFloat) withWidth;
- (NSImage *_Nullable) scaleToFitSize:(CGSize) withSize;
- (NSImage *_Nullable) scaleToFillSize:(CGSize) withSize;
- (Boolean)            isLandscape;
+ (NSImage *)          resizedImage:(NSImage *) sourceImage toPixelDimensions:(CGSize) newSize;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (Rotate)

+ (NSImage *) rotateImage:(NSImage *) image rotationInRadians:(CGFloat) rotationInRadians;
+ (NSImage *) rotateImage:(NSImage *) image rotationInDegrees:(CGFloat) rotationInDegrees;
+ (NSImage *) flipImageVertically:(NSImage *) image;
+ (NSImage *) flipImageHorizontally:(NSImage *) image;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
#ifndef TARGET_OS_IOS
@interface NSImage (CIImage)

+ (NSImage *_Nullable) imageWithCIImage:(CIImage *_Nullable) imageRef;

@end

#endif

NS_ASSUME_NONNULL_END

#endif
