#ifndef CKCoreObserver_h
#define CKCoreObserver_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #import <CodKits_macOS/CodKits_macOS.h>
#else
  #import <CodKits_iOS/CodKits_iOS.h>
#endif

#pragma mark Block Typedefs

typedef void(^CKCoreObserverBlockGeneric)          (id self, id newValue);
typedef void(^CKCoreObserverBlockChange)           (id self, id oldValue, id newValue);
typedef void(^CKCoreObserverBlockChangeMany)       (id self, NSString *keyPath, id oldValue, id newValue);
typedef void(^CKCoreObserverBlockInsert)           (id self, id newValue, NSIndexSet *indexes);
typedef void(^CKCoreObserverBlockRemove)           (id self, id oldValue,  NSIndexSet *indexes);
typedef void(^CKCoreObserverBlockReplace)          (id self, id oldValue, id newValue, NSIndexSet *indexes);

typedef void(^CKCoreObserverBlockForeignChange)    (id self, id object, id oldValue, id newValue);
typedef void(^CKCoreObserverBlockForeignChangeMany)(id self, id object, NSString *keyPath, id oldValue, id newValue);

typedef void(^CKCoreObserverBlockNotify)           (id self, NSNotification *notification);

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreObserver : NSObject

#pragma mark Initialization
/// Do not use. Observation target will be nil, so any calls to it will have no effect.
- (id)init;
/// Designated initializer.
- (id)initWithTarget:(NSObject *)target keyPath:(NSString *)keyPath owner:(id)owner;

#pragma mark Observation
/// Object that is observed when the receiver is attached.
@property (nonatomic, readonly, assign) id target;
/// Key-path that is observed on the target.
@property (nonatomic, readonly, copy) NSString *keyPath;
/// Object that 'owns' all blocks in this observer. This object was the caller of observation method.
@property (nonatomic, readonly, assign) id owner;


#pragma mark Attaching
/// Attached means, that this object really observes the key-path it was initialized with. Set it to add/remove this observer.
@property (nonatomic, readwrite, assign) BOOL attached;
/// Convenience method to set `attached` to YES.
- (void)attach;
/// Convenience method to set `attached` to NO.
- (void)detach;

#pragma mark Blocks
/// Add block to be executed on key-path setting of simple property or relationship.
- (void)addSettingObservationBlock:(CKCoreObserverBlockChange)block;
/// Add block to be executed on key-path relationship insertion.
- (void)addInsertionObservationBlock:(CKCoreObserverBlockInsert)block;
/// Append block to be executed on key-path relationship removal.
- (void)addRemovalObservationBlock:(CKCoreObserverBlockRemove)block;
/// Add block to be executed on key-path relationship replacement.
- (void)addReplacementObservationBlock:(CKCoreObserverBlockReplace)block;

@end

#endif

