#ifndef CKCoreSimpleTextField_h
#define CKCoreSimpleTextField_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <CodKits_iOS/CKCoreSimpleTextFieldKeyboard.h>

enum CKTextFieldValidationResult
  {
  CKCoreSimpleTextFieldValidationUnknown,
  CKCoreSimpleTextFieldValidationPassed,
  CKCoreSimpleTextFieldValidationFailed
  };

/**
 * Adds optional methods that are sent to the UITextFieldDelegate.
 */
//-------------------------------------------------------------------------------------------------------------------------
// PROTOCOL
//-------------------------------------------------------------------------------------------------------------------------
@protocol CKTextFieldValidationDelegate <NSObject>

@optional

- (void) textField:(id) aTextField validationResult:(enum CKTextFieldValidationResult) aResult forText:(NSString *) aText;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSimpleTextField: CKCoreSimpleTextFieldKeyboard

//
// User Defined Runtime Attributes (--> Storyboard!)
//
// *******************************************************
//                                                       *
//                                                       *

@property (nonatomic) IBInspectable NSString *validationType;
@property (nonatomic) IBInspectable NSString *minLength;
@property (nonatomic) IBInspectable NSString *maxLength;
@property (nonatomic) IBInspectable NSString *minValue;
@property (nonatomic) IBInspectable NSString *maxValue;
@property (nonatomic) IBInspectable NSString *pattern;

//                                                       *
//                                                       *
// *******************************************************

@property (nonatomic, weak) id<CKTextFieldValidationDelegate> validationDelegate;

/**
 * Set autocomplete values to enable autocompletion. Note that values will be
 * suggested in the order they are provided. Pass in a sorted array if you want
 * your values to be suggested in sorted order.
 * Please note, that inplace autocompletion will only work with left-aligned
 * text fields.
 */
@property (nonatomic) NSArray *autocompleteValues;

- (void) shake;
- (void) showAcceptButton;
- (void) hideAcceptButton;

@end

#endif

