#ifndef CKCoreTagList_h
#define CKCoreTagList_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

@class CKCoreTagList;
@class CKCoreTagListItem;

//-------------------------------------------------------------------------------------------------------------------------
// PROTOCOL
//-------------------------------------------------------------------------------------------------------------------------
@protocol CKCoreTagListDataSource <NSObject>

@required
- (NSUInteger)          numberOfItemsInTagList:(CKCoreTagList *) tagList;
- (CKCoreTagListItem *) tagList:(CKCoreTagList *) tagList tagItemAtIndex:(NSUInteger) tagItemIndex;

@optional
- (CGFloat) heightForAllItemsInTagList:(CKCoreTagList *) tagList;
- (CGFloat) horizontalSpaceBetweenItemsInTagList:(CKCoreTagList *) tagList;
- (CGFloat) verticalSpaceBetweenItemsInTagList:(CKCoreTagList *) tagList;

@end

//-------------------------------------------------------------------------------------------------------------------------
// PROTOCOL
//-------------------------------------------------------------------------------------------------------------------------
@protocol CKCoreTagListDelegate <NSObject>

@optional
- (void) tagList:(CKCoreTagList *) tagList itemTappedAtIndex:(NSUInteger) tagItemIndex;
- (void) tagList:(CKCoreTagList *) tagList itemAccessoryButtonTappedAtIndex:(NSUInteger) tagItemIndex;
- (void) tagList:(CKCoreTagList *) tagList requestedTagListItemRemovalAtIndex:(NSUInteger) tagItemIndex;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreTagList : UIView

@property (nonatomic, weak) id <CKCoreTagListDataSource> dataSource;
@property (nonatomic, weak) id <CKCoreTagListDelegate>   delegate;
@property (nonatomic, strong) UIView                    *tagListHeaderView;
@property (nonatomic, strong) UIView                    *tagListFooterView;
@property (nonatomic, assign) CGFloat                    leftEdgeInset;
@property (nonatomic, assign) CGFloat                    rightEdgeInset;

- (CKCoreTagListItem *) tagItemAtIndex:(NSUInteger) tagIndex;
- (void)                reloadData;

@end

#endif

