#ifndef CKCoreVideoTools_h
#define CKCoreVideoTools_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <AVFoundation/AVFoundation.h>

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreVideoTools

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreVideoResize : NSObject

- (void) proceed:(NSURL *) fromUrl toUrl:(NSURL *) toUrl withSize:(CGSize) withSize completionBlock:(void (^)(Boolean isSuccess, NSError *error)) completionBlock progressBlock:(void (^)(CGFloat progress)) progressBlock;
- (void) cleanUp;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreVideoCrop : NSObject

@property(nonatomic) NSString                 *videoExportQuality;
@property(nonatomic) AVFileType                videoExportFormat;

- (void) cropToSquare:(NSURL *) fromVideoURL toVideoURL:(NSURL *) toVideoURL toSquareWithSide:(CGFloat) sideLength completionBlock:(void (^)(Boolean isSuccess, NSError *error)) completionBlock progressBlock:(void (^)(CGFloat progress)) progressBlock;
- (void) cropToRectangle:(NSURL *) fromVideoURL toVideoURL:(NSURL *) toVideoURL toRectangleWithSide:(CGSize) sideLength completionBlock:(void (^)(Boolean isSuccess, NSError *error)) completionBlock progressBlock:(void (^)(CGFloat progress)) progressBlock;
- (void) cleanUp;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreVideoToolCreateVideoFromImages : NSObject

- (void) createVideoFromImagesAtUrl:(NSURL *) fileUrl withImages:(NSArray <NSImage *> *) withImages withSize:(CGSize) withSize completionBlock:(void (^)(Boolean isSuccess, NSError *error)) completionBlock progressBlock:(void (^)(CGFloat progress)) progressBlock;
- (void) cleanUp;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreVideoAddAudio : NSObject

@property(nonatomic) NSString                 *metadataAuthor;
@property(nonatomic) NSString                 *metadataTitle;
@property(nonatomic) NSImage                  *metadataThumbnailImage;
@property(nonatomic) NSString                 *metadataDescription;
@property(nonatomic) NSString                 *metadataCopyright;

@property(nonatomic) NSImage                  *videoWatermarkImage;
@property(nonatomic) CGFloat                   videoWatermarkImageAlpha;
@property(nonatomic) NSTimeInterval            videoFadeToBlackDuration;
@property(nonatomic) NSString                 *videoExportQuality;
@property(nonatomic) NSArray <NSDictionary *> *videoOriginalVolumes;
@property(nonatomic) NSArray <NSDictionary *> *audioOriginalVolumes;
@property(nonatomic) CGFloat                   videoExportSizePercentage;
@property(nonatomic) CGFloat                   videoExportCropRectangle;

- (void) proceed:(NSString *) withAudio withVideo:(NSString *) withVideo toDestination:(NSString *) toDestination completionBlock:(void (^)(Boolean isSuccess, NSError *error)) completionBlock progressBlock:(void (^)(CGFloat progress)) progressBlock;
- (void) proceed:(NSURL *) withAudioUrl withVideoUrl:(NSURL *) withVideoUrl toDestinationUrl:(NSURL *) toDestinationUrl completionBlock:(void (^)(Boolean isSuccess, NSError *error)) completionBlock progressBlock:(void (^)(CGFloat progress)) progressBlock;
- (void) proceed:(AVURLAsset *) withAudioAsset withVideoAsset:(AVURLAsset *) withVideoAsset toDestinationUrl:(NSURL *) toDestinationUrl completionBlock:(void (^)(Boolean isSuccess, NSError *error)) completionBlock progressBlock:(void (^)(CGFloat progress)) progressBlock;
- (void) cleanUp;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreVideoGetFrames : NSObject

- (void) startReadingAsset:(AVAsset *) asset completionBlock:(void (^)(Boolean isSuccess, NSError *error)) completionBlock progressBlock:(void (^)(CMSampleBufferRef bufferRef)) progressBlock;
- (void) cleanUp;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreVideoThumbnail : NSObject

+ (Boolean)   saveThumbnailAtTime:(NSURL *) fromVideoUrl atTime:(NSTimeInterval) atTime toUrl:(NSURL *) toUrl withMaxSize:(CGSize) withMaxSize;
+ (NSImage *) createThumbnail:(CMTime) atTime withAsset:(AVAsset *) withAsset withMaxSize:(CGSize) withMaxSize;
+ (NSImage *) getThumbnailAt:(CMTime) atTime withFile:(NSString *) withFile withMaxSize:(CGSize) withMaxSize;
+ (NSImage *) getThumbnailAt:(CMTime) atTime withURL:(NSURL *) withURL withMaxSize:(CGSize) withMaxSize;
+ (NSImage *) getThumbnailAt:(CMTime) atTime withAsset:(AVAsset *) withAsset withMaxSize:(CGSize) withMaxSize;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreVideoAddWatermark : NSObject

- (void) proceed:(NSURL *) fromUrl toUrl:(NSURL *) toUrl withWatermark:(NSImage *) withWatermark atLocation:(CGPoint) atLocation completionBlock:(void (^)(Boolean isSuccess, NSError *error)) completionBlock progressBlock:(void (^)(CGFloat progress)) progressBlock;
- (void) cleanUp;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreVideoAddOverlays : NSObject

- (void) proceed:(NSURL *) fromUrl toUrl:(NSURL *) toUrl withOverlays:(NSArray <NSDictionary *> *) withOverlays completionBlock:(void (^)(Boolean isSuccess, NSError *error)) completionBlock progressBlock:(void (^)(CGFloat progress)) progressBlock;
- (void) cleanUp;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreVideoAssembly : NSObject

@property(nonatomic) NSString                 *metadataAuthor;
@property(nonatomic) NSString                 *metadataTitle;
@property(nonatomic) NSImage                  *metadataThumbnailImage;
@property(nonatomic) NSString                 *metadataDescription;
@property(nonatomic) NSString                 *metadataCopyright;

@property(nonatomic) NSURL                    *videoSourceUrl;
@property(nonatomic) NSURL                    *videoOutputUrl;
@property(nonatomic) NSString                 *videoExportQuality;
@property(nonatomic) AVFileType                videoExportFormat;
@property(nonatomic) CGFloat                   videoExportSizePercentage;
@property(nonatomic) CGFloat                   videoAudioTrackVolume;
@property(nonatomic) NSArray <NSDictionary *> *videoCropAreas;
@property(nonatomic) NSArray <NSDictionary *> *videoWatermarkImages;
@property(nonatomic) NSArray <NSDictionary *> *videoAdditionalAudioTracks;

- (void) cleanUp;

@end

#endif

